
package com.annimon.ownlang;

import com.annimon.ownlang.lib.CallStack;
import com.annimon.ownlang.lib.Functions;
import com.annimon.ownlang.lib.Variables;
import com.annimon.ownlang.parser.Lexer;
import com.annimon.ownlang.parser.Parser;
import com.annimon.ownlang.parser.SourceLoader;
import com.annimon.ownlang.parser.ast.Statement;
import com.annimon.ownlang.parser.visitors.AssignValidator;
import com.annimon.ownlang.parser.visitors.FunctionAdder;
import java.util.Vector;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.StringItem;
import javax.microedition.lcdui.TextField;
import javax.microedition.midlet.MIDlet;

/**
 * @author aNNiMON
 */
public final class Main extends MIDlet implements CommandListener {
    
    private static final Command CMD_RUN = new Command("Запустить", Command.SCREEN, 1);
    private static final Command CMD_LOAD = new Command("Открыть", Command.OK, 2);
    private static final Command CMD_SHOW_TOKENS = new Command("Показать токены", Command.OK, 3);
    private static final Command CMD_SHOW_AST = new Command("Показать AST", Command.OK, 4);
    private static final Command CMD_CLEAR = new Command("Очистить консоль", Command.OK, 5);

    public static Main midlet;
    private final Display display;
    private final Form form;
    private final TextField input;
    private final StringItem console;
    private final FileBrowser browser;

    public Main() {
        midlet = Main.this;
        display = Display.getDisplay(Main.this);
        
        form = new Form("OwnLang v1.1.0");
        input = new TextField("Программа", SourceLoader.read("/program.own"), 8192, TextField.ANY);
        console = new StringItem("Консоль", "");
        console.setFont(Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_SMALL));
        
        browser = new FileBrowser();
        
        form.append(input);
        form.append(console);
        form.addCommand(CMD_RUN);
        form.addCommand(CMD_LOAD);
        form.addCommand(CMD_SHOW_TOKENS);
        form.addCommand(CMD_SHOW_AST);
        form.addCommand(CMD_CLEAR);
        form.setCommandListener(Main.this);
    }
    
    public Display getDisplay() {
        return display;
    }

    public void startApp() {
        display.setCurrent(form);
    }

    public void pauseApp() {
    }

    public void destroyApp(boolean ex) {
        notifyDestroyed();
    }

    public void commandAction(Command command, Displayable displayable) {
        if (command.equals(CMD_RUN)) {
            runProgram();
        } else if (command.equals(CMD_LOAD)) {
            showFileBrowser();
        } else if (command.equals(CMD_SHOW_TOKENS)) {
            try {
                final Vector/*<Token>*/ tokens = Lexer.tokenize(input.getString());
                for (int i = 0; i < tokens.size(); i++) {
                    Console.println(i + " " + tokens.elementAt(i));
                }
            } catch (Exception ex) {}
        } else if (command.equals(CMD_SHOW_AST)) {
            try {
                final Vector/*<Token>*/ tokens = Lexer.tokenize(input.getString());
                final Parser parser = new Parser(tokens);
                final Statement program = parser.parse();
                Console.println(program.toString());
            } catch (Exception ex) {}
        } else if (command.equals(CMD_CLEAR)) {
            clearConsole();
        }
    }

    private void runProgram() {
        Variables.clear();
        CallStack.clear();
        Functions.clear();
        try {
            final Vector/*<Token>*/ tokens = Lexer.tokenize(input.getString());
            final Parser parser = new Parser(tokens);
            final Statement program = parser.parse();
            if (parser.getParseErrors().hasErrors()) {
                Console.error(parser.getParseErrors().toString());
                return;
            }
            program.accept(new FunctionAdder());
            program.accept(new AssignValidator());
            try {
                program.execute();
            } catch (Exception exp) {
                exp.printStackTrace();
                Console.handleException(Thread.currentThread(), exp);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            Console.error("\r\n---\r\n" + ex.getMessage() + "\r\n---\r\n");
        }
    }
    
    public static void printToConsole(String text) {
        midlet.console.setText(midlet.console.getText() + text);
    }
    
    public static void clearConsole() {
        midlet.console.setText("");
    }
    
    public static void showInterpreter() {
        midlet.display.setCurrent(midlet.form);
    }
    
    public static void showInterpreter(String text) {
        midlet.input.setString(text);
        showInterpreter();
    }
    
    public static void showFileBrowser() {
        midlet.display.setCurrent(midlet.browser);
        midlet.browser.update();
    }
}
