package com.annimon.ownlang.lib.modules;

import com.annimon.ownlang.Main;
import com.annimon.ownlang.exceptions.ArgumentsMismatchException;
import com.annimon.ownlang.exceptions.TypeException;
import com.annimon.ownlang.lib.*;
import java.io.IOException;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.TiledLayer;

/**
 *
 * @author aNNiMON
 */
public final class canvas implements Module {
    
    private static final int CANVAS_IMAGE_TYPE = 786001;
    
    private static CanvasPanel panel;
    private static Graphics graphics;
    private static Image img;
    private static NumberValue lastKey, lastGA;
    
    private static Function onKeyPressed, onKeyReleased, onKeyRepeated;

    public void init() {
        Functions.set("showcanvas", new ShowCanvas());
        Functions.set("hidecanvas", new HideCanvas());
        Functions.set("keypressed", new KeyPressed());
        Functions.set("gameaction", new GameAction());
        Functions.set("setOnKeyPressed", new SetOnKeyPressed());
        Functions.set("setOnKeyReleased", new SetOnKeyReleased());
        Functions.set("setOnKeyRepeated", new SetOnKeyRepeated());
                
        Functions.set("createImage", new CreateImage());
        Functions.set("getImageSize", new getImageSize());
        
        Functions.set("tiledlayer", new CreateTiledLayer());
        Functions.set("setcell", new SetTiledLayerCell());
        Functions.set("drawtiledlayer", new DrawTiledLayer());
                
        Functions.set("line", new DrawLine());
        Functions.set("rect", new DrawRect());
        Functions.set("drawstring", new DrawString());
        Functions.set("drawImage", new DrawImage());
        Functions.set("frect", new FillRect());
        Functions.set("color", new SetColor());
        Functions.set("repaint", new Repaint());

        Variables.set("VK_UP", new NumberValue(Canvas.UP));
        Variables.set("VK_DOWN", new NumberValue(Canvas.DOWN));
        Variables.set("VK_LEFT", new NumberValue(Canvas.LEFT));
        Variables.set("VK_RIGHT", new NumberValue(Canvas.RIGHT));
        Variables.set("VK_FIRE", new NumberValue(Canvas.FIRE));
        
        lastKey = NumberValue.ZERO;
        lastGA = NumberValue.ZERO;
        
        panel = new CanvasPanel();
    }
    
    private static class CanvasPanel extends Canvas {
        
        public CanvasPanel() {
            setFullScreenMode(true);
            Variables.set("WIDTH", new NumberValue(getWidth()));
            Variables.set("HEIGHT", new NumberValue(getHeight()));
            img = Image.createImage(getWidth(), getHeight());
            graphics = img.getGraphics();
        }

        protected void paint(Graphics g) {
            g.drawImage(img, 0, 0, 20);
        }
        
        protected void keyPressed(int keyCode) {
            lastKey = new NumberValue(keyCode);
            lastGA = new NumberValue(getGameAction(keyCode));
            if (onKeyPressed != null) {
                onKeyPressed.execute(new Value[] {lastKey});
            }
        }
        
        protected void keyReleased(int keyCode) {
            lastKey = NumberValue.ZERO;
            lastGA = NumberValue.ZERO;
            if (onKeyReleased != null) {
                onKeyReleased.execute(new Value[] {new NumberValue(keyCode)});
            }
        }

        protected void keyRepeated(int keyCode) {
            if (onKeyRepeated != null) {
                onKeyRepeated.execute(new Value[] {new NumberValue(keyCode)});
            }
        }
    }
    
    private static class ImageValue implements Value {
        
        private final Image image;
        
        public ImageValue(Image image) {
            this.image = image;
        }
        
        public Object raw() {
            return image;
        }
        
        public int asInt() {
            throw new TypeException("Cannot cast Image to integer");
        }
        
        public double asNumber() {
            throw new TypeException("Cannot cast Image to number");
        }
        
        public String asString() {
            return image.toString();
        }
        
        public int type() {
            return CANVAS_IMAGE_TYPE;
        }
        
        public int compareTo(Object o) {
            // we don't need this
            return 0;
        }
        
        public String toString() {
            return "Image " + image;
        }
    }
    
    private static class CreateImage implements Function {

        public Value execute(Value[] args) {
            switch (args.length) {
                case 1: {
                    final String path = args[0].asString();
                    try {
                        return new ImageValue(Image.createImage(path));
                    } catch (IOException ex) {
                        throw new RuntimeException("Cannot open image " + path);
                    }
                }
                case 2:
                    return new ImageValue(Image.createImage(args[0].asInt(), args[1].asInt()));
                case 3: {
                    final ArrayValue array = (ArrayValue) args[0];
                    final int size = array.size();
                    final byte[] data = new byte[size];
                    for (int i = 0; i < size; i++) {
                        data[i] = (byte) array.get(i).asInt();
                    }
                    return new ImageValue(Image.createImage(data, args[1].asInt(), args[2].asInt()));
                }
                case 6:
                    return new ImageValue(Image.createImage(
                            ((ImageValue) args[0]).image,
                            args[1].asInt(), args[2].asInt(),
                            args[3].asInt(), args[4].asInt(),
                            args[5].asInt()));
            }
            throw new ArgumentsMismatchException();
        }
    }
    
    private static class getImageSize implements Function {

        public Value execute(Value[] args) {
            final Image image = ((ImageValue) args[0]).image;
            return new ArrayValue(new Value[] {
                new NumberValue(image.getWidth()),
                new NumberValue(image.getHeight())
            });
        }
    }
    
    private static class ShowCanvas implements Function {

        public Value execute(Value[] args) {
            panel = new CanvasPanel();
            Main.midlet.getDisplay().setCurrent(panel);
            return NumberValue.ZERO;
        }
    }
    
    private static class HideCanvas implements Function {

        public Value execute(Value[] args) {
            Main.showInterpreter();
            return NumberValue.ZERO;
        }
    }
    
    private static class SetOnKeyPressed implements Function {

        public Value execute(Value[] args) {
            onKeyPressed = ((FunctionValue) args[0]).getValue();
            return NumberValue.ZERO;
        }
    }
    private static class SetOnKeyReleased implements Function {

        public Value execute(Value[] args) {
            onKeyReleased = ((FunctionValue) args[0]).getValue();
            return NumberValue.ZERO;
        }
    }
    private static class SetOnKeyRepeated implements Function {

        public Value execute(Value[] args) {
            onKeyRepeated = ((FunctionValue) args[0]).getValue();
            return NumberValue.ZERO;
        }
    }
    
    private static class KeyPressed implements Function {
        public Value execute(Value[] args) {
            return lastKey;
        }
    }
    private static class GameAction implements Function {
        public Value execute(Value[] args) {
            return lastGA;
        }
    }
    
    private static class DrawLine implements Function {

        public Value execute(Value[] args) {
            int x1 = args[0].asInt();
            int y1 = args[1].asInt();
            int x2 = args[2].asInt();
            int y2 = args[3].asInt();
            graphics.drawLine(x1, y1, x2, y2);
            return NumberValue.ZERO;
        }
    }
    
    private static class DrawRect implements Function {

        public Value execute(Value[] args) {
            int x = args[0].asInt();
            int y = args[1].asInt();
            int w = args[2].asInt();
            int h = args[3].asInt();
            graphics.drawRect(x, y, w, h);
            return NumberValue.ZERO;
        }
    }
    
    private static class DrawString implements Function {

        public Value execute(Value[] args) {
            String text = args[0].asString();
            int x = args[1].asInt();
            int y = args[2].asInt();
            int anchor = args.length >= 4 ? args[3].asInt() : (Graphics.TOP | Graphics.LEFT);
            graphics.drawString(text, x, y, anchor);
            return NumberValue.ZERO;
        }
    }
    
    private static class DrawImage implements Function {

        public Value execute(Value[] args) {
            Image img = ((ImageValue) args[0]).image;
            int x = args[1].asInt();
            int y = args[2].asInt();
            int anchor = args.length >= 4 ? args[3].asInt() : (Graphics.TOP | Graphics.LEFT);
            graphics.drawImage(img, x, y, anchor);
            return NumberValue.ZERO;
        }
    }
    
    private static class FillRect implements Function {

        public Value execute(Value[] args) {
            int x = args[0].asInt();
            int y = args[1].asInt();
            int w = args[2].asInt();
            int h = args[3].asInt();
            graphics.fillRect(x, y, w, h);
            return NumberValue.ZERO;
        }
    }

    private static class SetColor implements Function {

        public Value execute(Value[] args) {
            if (args.length == 1) {
                graphics.setColor(args[0].asInt());
                return NumberValue.ZERO;
            }
            int r = args[0].asInt();
            int g = args[1].asInt();
            int b = args[2].asInt();
            graphics.setColor(r, g, b);
            return NumberValue.ZERO;
        }

    }

    private static class Repaint implements Function {

        public Value execute(Value[] args) {
            panel.repaint();
            return NumberValue.ZERO;
        }
    }
    
    private static TiledLayer tiledlayer;
    private static class CreateTiledLayer implements Function {

        public Value execute(Value[] args) {
            final int columns = args[0].asInt();
            final int rows = args[1].asInt();
            Image img = ((ImageValue) args[2]).image;
            final int tileWidth = args[3].asInt();
            final int tileHeight = args[4].asInt();
            tiledlayer = new TiledLayer(columns, rows, img, tileWidth, tileHeight);
            return NumberValue.ZERO;
        }
    }
    
    private static class SetTiledLayerCell implements Function {

        public Value execute(Value[] args) {
            final int col = args[0].asInt();
            final int row = args[1].asInt();
            final int tileIndex = args[2].asInt();
            tiledlayer.setCell(col, row, tileIndex);
            return NumberValue.ZERO;
        }
    }
    
    private static class DrawTiledLayer implements Function {

        public Value execute(Value[] args) {
            tiledlayer.paint(graphics);
            return NumberValue.ZERO;
        }
    }
}