package com.annimon.ownlang.lib.modules;

import com.annimon.ownlang.exceptions.ArgumentsMismatchException;
import com.annimon.ownlang.lib.*;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.microedition.io.Connector;
import javax.microedition.io.file.FileConnection;

/**
 *
 * @author aNNiMON
 */
public final class files implements Module {

    private static Hashtable/*<Integer, FileInfo>*/ files;
    
    /*@Override*/
    public void init() {
        files = new Hashtable/*<>*/();
        
        Functions.set("fopen", new fopen());
        Functions.set("listFiles", new listFiles());
        Functions.set("delete", new delete());
        Functions.set("exists", new exists());
        Functions.set("isDirectory", new isDirectory());
        Functions.set("isFile", new isFile());
        Functions.set("mkdir", new mkdir());
        Functions.set("fileSize", new fileSize());
        Functions.set("readBoolean", new readBoolean());
        Functions.set("readByte", new readByte());
        Functions.set("readBytes", new readBytes());
        Functions.set("readAllBytes", new readAllBytes());
        Functions.set("readChar", new readChar());
        Functions.set("readShort", new readShort());
        Functions.set("readInt", new readInt());
        Functions.set("readLong", new readLong());
        Functions.set("readFloat", new readFloat());
        Functions.set("readDouble", new readDouble());
        Functions.set("readUTF", new readUTF());
        Functions.set("readLine", new readLine());
        Functions.set("readText", new readText());
        Functions.set("writeBoolean", new writeBoolean());
        Functions.set("writeByte", new writeByte());
        Functions.set("writeChar", new writeChar());
        Functions.set("writeShort", new writeShort());
        Functions.set("writeInt", new writeInt());
        Functions.set("writeLong", new writeLong());
        Functions.set("writeFloat", new writeFloat());
        Functions.set("writeDouble", new writeDouble());
        Functions.set("writeUTF", new writeUTF());
        Functions.set("writeLine", new writeLine());
        Functions.set("writeText", new writeText());
        Functions.set("flush", new flush());
        Functions.set("fclose", new fclose());
    }
    
    private static class fopen implements Function {

        /*@Override*/
        public Value execute(Value[] args) {
            if (args.length < 1) throw new ArgumentsMismatchException("At least one argument expected");
            
            try {
                final FileConnection file = (FileConnection) Connector.open(args[0].asString());
                if (args.length > 1) {
                    return process(file, args[1].asString().toLowerCase());
                }
                return process(file, "r");
            } catch (IOException ioe) {
                return NumberValue.MINUS_ONE;
            }
        }
        
        private Value process(FileConnection file, String mode) throws IOException {
            DataInputStream dis = null;
            InputStreamReader reader = null;
            if (mode.indexOf("rb") != -1) {
                dis = file.openDataInputStream();
            } else if (mode.indexOf("r") != -1) {
                reader = new InputStreamReader(file.openInputStream(), "UTF-8");
            }
            
            DataOutputStream dos = null;
            OutputStreamWriter writer = null;
            if (mode.indexOf("wb") != -1) {
                dos = file.openDataOutputStream();
            } else if (mode.indexOf("w") != -1) {
                writer = new OutputStreamWriter(file.openOutputStream(), "UTF-8");
            }
            
            final int key = files.size();
            files.put(new Integer(key), new FileInfo(file, dis, dos, reader, writer));
            return new NumberValue(key);
        }
    }
    
    private static abstract class FileFunction implements Function {
        
        /*@Override*/
        public Value execute(Value[] args) {
            if (args.length < 1) throw new ArgumentsMismatchException("File descriptor expected");
            final int key = args[0].asInt();
            try {
                return execute((FileInfo)files.get(new Integer(key)), args);
            } catch (IOException ioe) {
                return NumberValue.ZERO;
            }
        }
        
        protected abstract Value execute(FileInfo fileInfo, Value[] args) throws IOException;
    }
    
    private static class listFiles extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            final Vector vector = new Vector();
            for (Enumeration en = fileInfo.file.list(); en.hasMoreElements();) {
                vector.addElement(new StringValue((String)en.nextElement()));
            }
            return new ArrayValue(vector);
        }
    }
    
    private static class delete extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            fileInfo.file.delete();
            return NumberValue.ONE;
        }
    }
    
    private static class exists extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            return NumberValue.fromBoolean(fileInfo.file.exists());
        }
    }
    
    private static class isDirectory extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            return NumberValue.fromBoolean(fileInfo.file.isDirectory());
        }
    }
    
    private static class isFile extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            return NumberValue.fromBoolean(!fileInfo.file.isDirectory());
        }
    }
    
    private static class mkdir extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            fileInfo.file.mkdir();
            return NumberValue.ONE;
        }
    }
    
    private static class fileSize extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            return new NumberValue(fileInfo.file.fileSize());
        }
    }
    
    private static class readBoolean extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            return NumberValue.fromBoolean(fileInfo.dis.readBoolean());
        }
    }
    
    private static class readByte extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            return new NumberValue(fileInfo.dis.readByte());
        }
    }
    
    private static class readBytes extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            final ArrayValue array = (ArrayValue) args[1];
            int offset = 0, length = array.size();
            if (args.length > 3) {
                offset = args[2].asInt();
                length = args[3].asInt();
            }
            
            final byte[] buffer = new byte[length];
            final int readed = fileInfo.dis.read(buffer, offset, length);
            for (int i = 0; i < readed; i++) {
                array.set(i, new NumberValue(buffer[i]));
            }
            return new NumberValue(readed);
        }
    }
    
    private static class readAllBytes extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            final int bufferSize = 4096;
            final byte[] buffer = new byte[bufferSize];
            final ByteArrayOutputStream baos = new ByteArrayOutputStream();
            int readed;
            while ((readed = fileInfo.dis.read(buffer, 0, bufferSize)) != -1) {
                baos.write(buffer, 0, readed);
            }
            baos.flush();
            baos.close();
            final byte[] bytes = baos.toByteArray();
            final int size = bytes.length;
            final ArrayValue result = new ArrayValue(size);
            for (int i = 0; i < size; i++) {
                result.set(i, new NumberValue(bytes[i]));
            }
            return result;
        }
    }
    
    private static class readChar extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            return new NumberValue((short)fileInfo.dis.readChar());
        }
    }
    
    private static class readShort extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            return new NumberValue(fileInfo.dis.readShort());
        }
    }
    
    private static class readInt extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            return new NumberValue(fileInfo.dis.readInt());
        }
    }
    
    private static class readLong extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            return new NumberValue(fileInfo.dis.readLong());
        }
    }
    
    private static class readFloat extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            return new NumberValue(fileInfo.dis.readFloat());
        }
    }
    
    private static class readDouble extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            return new NumberValue(fileInfo.dis.readDouble());
        }
    }
    
    private static class readUTF extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            return new StringValue(fileInfo.dis.readUTF());
        }
    }
    
    private static class readLine extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            final StringBuffer sb = new StringBuffer();
            int ch;
            while ((ch = fileInfo.reader.read()) != -1) {
                sb.append((char) ch);
                if (ch == '\n') break;
            }
            return new StringValue(sb.toString());
        }
    }
    
    private static class readText extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            final StringBuffer sb = new StringBuffer();
            int ch;
            while ((ch = fileInfo.reader.read()) != -1) {
                sb.append((char) ch);
            }
            return new StringValue(sb.toString());
        }
    }
    
    
    private static class writeBoolean extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            fileInfo.dos.writeBoolean(args[1].asInt() != 0);
            return NumberValue.ONE;
        }
    }
    
    private static class writeByte extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            fileInfo.dos.writeByte((byte) args[1].asInt());
            return NumberValue.ONE;
        }
    }
    
    private static class writeChar extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            final char ch = (args[1].type() == Types.NUMBER)
                    ? ((char) args[1].asInt())
                    : args[1].asString().charAt(0);
            fileInfo.dos.writeChar(ch);
            return NumberValue.ONE;
        }
    }
    
    private static class writeShort extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            fileInfo.dos.writeShort((short) args[1].asInt());
            return NumberValue.ONE;
        }
    }
    
    private static class writeInt extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            fileInfo.dos.writeInt(args[1].asInt());
            return NumberValue.ONE;
        }
    }
    
    private static class writeLong extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            fileInfo.dos.writeLong((long) args[1].asNumber());
            return NumberValue.ONE;
        }
    }
    
    private static class writeFloat extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            fileInfo.dos.writeFloat((float) args[1].asNumber());
            return NumberValue.ONE;
        }
    }
    
    private static class writeDouble extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            fileInfo.dos.writeDouble(args[1].asNumber());
            return NumberValue.ONE;
        }
    }
    
    private static class writeUTF extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            fileInfo.dos.writeUTF(args[1].asString());
            return NumberValue.ONE;
        }
    }
    
    private static class writeLine extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            fileInfo.writer.write(args[1].asString());
            fileInfo.writer.write("\n");
            return NumberValue.ONE;
        }
    }
    
    private static class writeText extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            fileInfo.writer.write(args[1].asString());
            return NumberValue.ONE;
        }
    }
    
    private static class flush extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            if (fileInfo.dos != null) {
                fileInfo.dos.flush();
            }
            if (fileInfo.writer != null) {
                fileInfo.writer.flush();
            }
            return NumberValue.ONE;
        }
    }
    
    private static class fclose extends FileFunction {
        /*@Override*/
        protected Value execute(FileInfo fileInfo, Value[] args) throws IOException {
            if (fileInfo.dis != null) {
                fileInfo.dis.close();
            }
            if (fileInfo.dos != null) {
                fileInfo.dos.close();
            }
            if (fileInfo.reader != null) {
                fileInfo.reader.close();
            }
            if (fileInfo.writer != null) {
                fileInfo.writer.close();
            }
            return NumberValue.ONE;
        }
    }
    
    private static class FileInfo {
        FileConnection file;
        DataInputStream dis;
        DataOutputStream dos;
        InputStreamReader reader;
        OutputStreamWriter writer;

        public FileInfo(FileConnection file, DataInputStream dis, DataOutputStream dos, InputStreamReader reader, OutputStreamWriter writer) {
            this.file = file;
            this.dis = dis;
            this.dos = dos;
            this.reader = reader;
            this.writer = writer;
        }
    }
}
