package com.annimon.ownlang.lib.modules.functions;

import com.annimon.ownlang.exceptions.ArgumentsMismatchException;
import com.annimon.ownlang.exceptions.TypeException;
import com.annimon.ownlang.lib.*;
import com.annimon.stream.Entry;
import java.util.Enumeration;
import java.util.Vector;

public final class functional_filter implements Function {

    /*@Override*/
    public Value execute(Value[] args) {
        if (args.length < 2) throw new ArgumentsMismatchException("At least two args expected");
        
        if (args[1].type() != Types.FUNCTION) {
            throw new TypeException("Function expected in second arg");
        }
        final Value container = args[0];
        final Function consumer = ((FunctionValue) args[1]).getValue();
        if (container.type() == Types.ARRAY) {
            return filterArray((ArrayValue) container, consumer);
        }
        
        if (container.type() == Types.MAP) {
            return filterMap((MapValue) container, consumer);
        }

        throw new TypeException("Invalid first argument. Array or map expected");
    }
    
    private Value filterArray(ArrayValue array, Function predicate) {
        final int size = array.size();
        final Vector/*<Value>*/ values = new Vector/*<Value>*/(size);
        for (Enumeration en = array.iterator(); en.hasMoreElements();) {
            final Value value = (Value) en.nextElement();
            if (predicate.execute(new Value[]{value}) != NumberValue.ZERO) {
                values.addElement(value);
            }
        }
        final int newSize = values.size();
        final Value[] results = new Value[newSize];
        values.copyInto(results);
        return new ArrayValue(results);
    }
    
    private Value filterMap(MapValue map, Function predicate) {
        final MapValue result = new MapValue(map.size());
        for (Enumeration en = map.iterator(); en.hasMoreElements();) {
            final Entry element = (Entry) en.nextElement();
            if (predicate.execute(new Value[]{(Value)element.getKey(), (Value)element.getValue()}) != NumberValue.ZERO) {
                result.set((Value)element.getKey(), (Value)element.getValue());
            }
        }
        return result;
    }
}