package com.annimon.ownlang.lib.modules;

import com.annimon.ownlang.lib.*;
import com.annimon.stream.Collectors;
import com.annimon.stream.Entry;
import com.annimon.stream.Objects;
import com.annimon.stream.Stream;
import com.annimon.stream.function.Predicate;
import java.util.Enumeration;
import java.util.Vector;

/**
 *
 * @author aNNiMON
 */
public final class ounit implements Module {
    
    public static final Value[] EMPTY = new Value[0];

    /*@Override*/
    public void init() {
        Functions.set("assertEquals", new assertEquals());
        Functions.set("assertNotEquals", new assertNotEquals());
        Functions.set("assertSameType", new assertSameType());
        Functions.set("assertTrue", new assertTrue());
        Functions.set("assertFalse", new assertFalse());
        Functions.set("runTests", new runTests());
    }
    
    private static String millisToSeconds(long millis) {
        return (millis / 1000d) + " sec";
    }
    
    private static class assertEquals implements Function {
        /*@Override*/
        public Value execute(Value[] args) {
            if (args[0].equals(args[1])) return NumberValue.ONE;
            throw new OUnitAssertionException("Values are not equals: "
                    + "1: " + args[0] + ", 2: " + args[1]);
        }
    }
    
    private static class assertNotEquals implements Function {
        /*@Override*/
        public Value execute(Value[] args) {
            if (!args[0].equals(args[1])) return NumberValue.ONE;
            throw new OUnitAssertionException("Values are equals: " + args[0]);
        }
    }
    
    private static class assertSameType implements Function {
        /*@Override*/
        public Value execute(Value[] args) {
            if (args[0].type() == args[1].type()) return NumberValue.ONE;
            throw new OUnitAssertionException("Types mismatch. "
                    + "1: " + Types.typeToString(args[0].type())
                    + ", 2: " + Types.typeToString(args[1].type()));
        }
    }
    
    private static class assertTrue implements Function {
        /*@Override*/
        public Value execute(Value[] args) {
            if (args[0].asInt() != 0) return NumberValue.ONE;
            throw new OUnitAssertionException("Expected true, but found false.");
        }
    }
    
    private static class assertFalse implements Function {
        /*@Override*/
        public Value execute(Value[] args) {
            if (args[0].asInt() == 0) return NumberValue.ONE;
            throw new OUnitAssertionException("Expected false, but found true.");
        }
    }
    
    private static class runTests implements Function {
        
        /*@Override*/
        public Value execute(Value[] args) {
            Vector/*<TestInfo>*/ tests = (Vector) Stream.of(Objects.entrySet(Functions.getFunctions()))
                    .filter(new Predicate() {
                        public boolean test(Object value) {
                            final Entry e = ((Entry) value);
                            final String funcName = (String) e.getKey();
                            return funcName.startsWith("test");
                        }
                    })
                    .map(new com.annimon.stream.function.Function() {
                        public Object apply(Object value) {
                            final Entry e = ((Entry) value);
                            return runTest((String) e.getKey(), (Function) e.getValue());
                        }
                    })
                    .collect(Collectors.toList());
            
            int failures = 0;
            long summaryTime = 0;
            final StringBuffer result = new StringBuffer();
            for (Enumeration en = tests.elements(); en.hasMoreElements();) {
                final TestInfo test = (TestInfo) en.nextElement();
                if (!test.isPassed) failures++;
                summaryTime += test.elapsedTimeInMillis;
                result.append("\n");
                result.append(test.info());
            }
            result.append("\n");
            result.append("Tests run: ").append(tests.size())
                  .append(", Failures: ").append(failures)
                  .append(", Time elapsed: ").append(millisToSeconds(summaryTime));
            return new StringValue(result.toString());
        }
        
        private TestInfo runTest(String name, Function f) {
            final long startTime = System.currentTimeMillis();
            boolean isSuccessfull;
            String failureDescription;
            try {
                f.execute(EMPTY);
                isSuccessfull = true;
                failureDescription = "";
            } catch (OUnitAssertionException oae) {
                isSuccessfull = false;
                failureDescription = oae.getMessage();
            }
            final long elapsedTime = System.currentTimeMillis() - startTime;
            return new TestInfo(name, isSuccessfull, failureDescription, elapsedTime / 1000);
        }
    }
    
    private static class OUnitAssertionException extends RuntimeException {

        public OUnitAssertionException(String message) {
            super(message);
        }
    }
    
    private static class TestInfo {
        String name;
        boolean isPassed;
        String failureDescription;
        long elapsedTimeInMillis;

        public TestInfo(String name, boolean isPassed, String failureDescription, long elapsedTimeInMillis) {
            this.name = name;
            this.isPassed = isPassed;
            this.failureDescription = failureDescription;
            this.elapsedTimeInMillis = elapsedTimeInMillis;
        }
        
        public String info() {
            return new StringBuffer()
                    .append(name)
                    .append(" [").append(isPassed ? "passed" : "FAILED").append("]\n")
                    .append(isPassed ? "" : (failureDescription + "\n"))
                    .append("Elapsed: ").append(millisToSeconds(elapsedTimeInMillis)).append('\n')
                    .toString();
        }
    }
}
