package com.annimon.ownlang.parser.visitors;

import com.annimon.ownlang.parser.ast.*;
import com.annimon.stream.Entry;
import com.annimon.stream.Objects;
import java.util.Enumeration;

/**
 *
 * @author aNNiMON
 */
public abstract class AbstractVisitor implements Visitor {

    /*@Override*/
    public void visit(ArrayExpression s) {
        for (Enumeration en = s.elements.elements(); en.hasMoreElements();) {
            ((Expression) en.nextElement()).accept(this);
        }
    }

    /*@Override*/
    public void visit(AssignmentExpression s) {
        s.expression.accept(this);
    }

    /*@Override*/
    public void visit(BinaryExpression s) {
        s.expr1.accept(this);
        s.expr2.accept(this);
    }

    /*@Override*/
    public void visit(BlockStatement s) {
        for (Enumeration en = s.statements.elements(); en.hasMoreElements();) {
            ((Statement) en.nextElement()).accept(this);
        }
    }

    /*@Override*/
    public void visit(BreakStatement s) {
    }

    /*@Override*/
    public void visit(ConditionalExpression s) {
        s.expr1.accept(this);
        s.expr2.accept(this);
    }
    
    /*@Override*/
    public void visit(ContainerAccessExpression s) {
        for (Enumeration en = s.indices.elements(); en.hasMoreElements();) {
            ((Expression) en.nextElement()).accept(this);
        }
    }

    /*@Override*/
    public void visit(ContinueStatement s) {
    }
    
    /*@Override*/
    public void visit(DestructuringAssignmentStatement s) {
        s.containerExpression.accept(this);
    }

    /*@Override*/
    public void visit(DoWhileStatement s) {
        s.condition.accept(this);
        s.statement.accept(this);
    }

    /*@Override*/
    public void visit(ForStatement s) {
        s.initialization.accept(this);
        s.termination.accept(this);
        s.increment.accept(this);
        s.statement.accept(this);
    }
    
    /*@Override*/
    public void visit(ForeachArrayStatement s) {
        s.container.accept(this);
        s.body.accept(this);
    }
    
    /*@Override*/
    public void visit(ForeachMapStatement s) {
        s.container.accept(this);
        s.body.accept(this);
    }

    /*@Override*/
    public void visit(FunctionDefineStatement s) {
        s.body.accept(this);
    }

    /*@Override*/
    public void visit(FunctionReferenceExpression e) {
    }

    /*@Override*/
    public void visit(ExprStatement s) {
        s.expr.accept(this);
    }

    /*@Override*/
    public void visit(FunctionalExpression s) {
        s.functionExpr.accept(this);
        for (Enumeration en = s.arguments.elements(); en.hasMoreElements();) {
            ((Expression) en.nextElement()).accept(this);
        }
    }

    /*@Override*/
    public void visit(IfStatement s) {
        s.expression.accept(this);
        s.ifStatement.accept(this);
        if (s.elseStatement != null) {
            s.elseStatement.accept(this);
        }
    }
    
    /*@Override*/
    public void visit(IncludeStatement s) {
        s.expression.accept(this);
    }
    
    /*@Override*/
    public void visit(MapExpression s) {
        for (Enumeration en = Objects.entrySet(s.elements).elements(); en.hasMoreElements();) {
            final Entry entry = (Entry) en.nextElement();
            ((Expression)entry.getKey()).accept(this);
            ((Expression)entry.getValue()).accept(this);
        }
    }
    
    /*@Override*/
    public void visit(MatchExpression s) {
        s.expression.accept(this);
    }

    /*@Override*/
    public void visit(PrintStatement s) {
        s.expression.accept(this);
    }
    
    /*@Override*/
    public void visit(PrintlnStatement s) {
        s.expression.accept(this);
    }

    /*@Override*/
    public void visit(ReturnStatement s) {
        s.expression.accept(this);
    }

    /*@Override*/
    public void visit(TernaryExpression s) {
        s.condition.accept(this);
        s.trueExpr.accept(this);
        s.falseExpr.accept(this);
    }
    
    /*@Override*/
    public void visit(UnaryExpression s) {
        s.expr1.accept(this);
    }

    /*@Override*/
    public void visit(ValueExpression s) {
    }

    /*@Override*/
    public void visit(VariableExpression s) {
    }

    /*@Override*/
    public void visit(WhileStatement st) {
        st.condition.accept(this);
        st.statement.accept(this);
    }

    /*@Override*/
    public void visit(UseStatement st) {
        st.expression.accept(this);
    }
}